==> RSTouchscreenNG-1.3.0.zip: RSTouchscreenNG/projects/front-ui/src/lib/drag.directive.ts <==
/* (C) 2016-2021 Hot-World GmbH & Co. KG <rlittwin@repetier.com>
 All Rights Reserved

 THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND
 ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
 WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED.
 IN NO EVENT SHALL THE COPYRIGHT HOLDER OR CONTRIBUTORS BE LIABLE FOR ANY
 DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
 (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
 LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
 ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
 SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.

 It is allowed to use this code to create or use in products connecting to
 Repetier-Server or to use it for own plugins/modules with Repetier-Server.
 This includes free and commercial products.
 Usage for other products is strictly forbidden.
 */

import {Directive, ElementRef, HostBinding, HostListener, OnDestroy, OnInit} from '@angular/core';

@Directive({
    selector: '[rsDrag]'
})
export class RSDragDirective implements OnInit, OnDestroy {
    static lastDragEnd = 0;
    static dragMinimum = 25;

    @HostBinding('style.overflow') styleOverflow = 'hidden';
    x: number;
    y: number;
    top: number;
    left: number;
    down = false;
    moved = false;
    touch = false;

    constructor(public elem: ElementRef) {
    }

    ngOnInit() {
        document.body.addEventListener('mousemove', this.onMousemove);
        document.body.addEventListener('touchmove', this.onTouchmove);
        document.body.addEventListener('mouseup', this.onMouseup);
        document.body.addEventListener('touchend', this.onMouseup);
        document.body.addEventListener('mouseleave', this.onMouseup);
        document.body.addEventListener('touchstart', this.touchstart, true);
        this.elem.nativeElement.addEventListener('click', this.onClick, true);
        // this.elem.nativeElement.addEventListener("touchstart", this.touchstart, true);
    }

    ngOnDestroy() {
        document.body.removeEventListener('mousemove', this.onMousemove);
        document.body.removeEventListener('touchmove', this.onMousemove);
        document.body.removeEventListener('mouseup', this.onMouseup);
        document.body.removeEventListener('touchend', this.onMouseup);
        document.body.removeEventListener('mouseleave', this.onMouseup);
        document.body.removeEventListener('touchstart', this.touchstart);
        this.elem.nativeElement.removeEventListener('click', this.onClick);
    }

    touchstart = (e: any) => {
        let act: Element = e.target as Element;
        while (act !== this.elem.nativeElement && act) {
            act = act.parentElement;
        }
        if (act !== this.elem.nativeElement) {
            return;
        }
        this.down = true;
        this.moved = false;
        this.x = (e.touches ? e.touches[0].pageX : e.pageX);
        this.y = (e.touches ? e.touches[0].pageY : e.pageY);
        this.top = this.elem.nativeElement.scrollTop;
        this.left = this.elem.nativeElement.scrollLeft;
        this.touch = true;
        // e.preventDefault(); // prevents click/hit from working
        // e.stopPropagation();
    }

    @HostListener('mousedown', ['$event']) onMousedown = (e: any) => {
        if (this.touch) {
            return;
        }
        this.down = true;
        this.moved = false;
        this.x = (e.touches ? e.touches[0].pageX : e.pageX);
        this.y = (e.touches ? e.touches[0].pageY : e.pageY);
        this.top = this.elem.nativeElement.scrollTop;
        this.left = this.elem.nativeElement.scrollLeft;
    }

    onClick = (e) => {
        if (this.moved) {
            e.stopPropagation();
            e.preventDefault();
        }
    }
    onTouchmove = (e: any) => {
        if (this.down) {
            // this.elem.nativeElement.style.backgroundColor = "#ff00ff";
            const newX = (e.touches ? e.touches[0].pageX : e.pageX);
            const newY = (e.touches ? e.touches[0].pageY : e.pageY);
            const dx = Math.abs(this.x - newX);
            const dy = Math.abs(this.y - newY);
            if (!this.moved) {
                if (dx < RSDragDirective.dragMinimum && dy < RSDragDirective.dragMinimum) {
                    return;
                }
            }
            this.elem.nativeElement.scrollTop = (this.top - newY + this.y);
            this.elem.nativeElement.scrollLeft = (this.left - newX + this.x);
            this.moved = true;
        }
    }

    onMousemove = (e: any) => {
        if (this.touch) {
            return;
        }
        if (this.down) {
            // this.elem.nativeElement.style.backgroundColor = "#0000ff";
            const newX = (e.touches ? e.touches[0].pageX : e.pageX);
            const newY = (e.touches ? e.touches[0].pageY : e.pageY);
            const dx = Math.abs(this.x - newX);
            const dy = Math.abs(this.y - newY);
            if (!this.moved) {
                if (dx < RSDragDirective.dragMinimum && dy < RSDragDirective.dragMinimum) {
                    return;
                }
            }
            this.elem.nativeElement.scrollTop = (this.top - newY + this.y);
            this.elem.nativeElement.scrollLeft = (this.left - newX + this.x);
            this.moved = true;
        }
    }

    onMouseup = (e) => {
        if (this.moved) {
            RSDragDirective.lastDragEnd = new Date().getTime();
            this.moved = false;
        }
        this.down = false;
        this.touch = false;
        this.elem.nativeElement.style.backgroundColor = "transparent";
    }
}

==> RSTouchscreenNG-1.4.1.zip: RSTouchscreenNG/projects/front-ui/src/lib/drag.directive.ts <==
/* (C) 2016-2021 Hot-World GmbH & Co. KG <rlittwin@repetier.com>
 All Rights Reserved

 THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND
 ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
 WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED.
 IN NO EVENT SHALL THE COPYRIGHT HOLDER OR CONTRIBUTORS BE LIABLE FOR ANY
 DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
 (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
 LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
 ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
 SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.

 It is allowed to use this code to create or use in products connecting to
 Repetier-Server or to use it for own plugins/modules with Repetier-Server.
 This includes free and commercial products.
 Usage for other products is strictly forbidden.
 */

import {Directive, ElementRef, HostBinding, HostListener, OnDestroy, OnInit} from '@angular/core';

@Directive({
    selector: '[rsDrag]'
})
export class RSDragDirective implements OnInit, OnDestroy {
    static lastDragEnd = 0;
    static dragMinimum = 25;

    // @HostBinding('style.overflow') styleOverflow = 'hidden';
    @HostBinding('class.rsdrag') classRsdrag = true;
    x: number;
    y: number;
    top: number;
    left: number;
    down = false;
    moved = false;
    touch = false;

    constructor(public elem: ElementRef) {
    }

    ngOnInit() {
        document.body.addEventListener('mousemove', this.onMousemove);
        document.body.addEventListener('touchmove', this.onTouchmove);
        document.body.addEventListener('mouseup', this.onMouseup);
        document.body.addEventListener('touchend', this.onMouseup);
        document.body.addEventListener('mouseleave', this.onMouseup);
        document.body.addEventListener('touchstart', this.touchstart, true);
        this.elem.nativeElement.addEventListener('click', this.onClick, true);
        // this.elem.nativeElement.addEventListener("touchstart", this.touchstart, true);
    }

    ngOnDestroy() {
        document.body.removeEventListener('mousemove', this.onMousemove);
        document.body.removeEventListener('touchmove', this.onMousemove);
        document.body.removeEventListener('mouseup', this.onMouseup);
        document.body.removeEventListener('touchend', this.onMouseup);
        document.body.removeEventListener('mouseleave', this.onMouseup);
        document.body.removeEventListener('touchstart', this.touchstart);
        this.elem.nativeElement.removeEventListener('click', this.onClick);
    }

    touchstart = (e: any) => {
        let act: Element = e.target as Element;
        while (act !== this.elem.nativeElement && act) {
            act = act.parentElement;
        }
        if (act !== this.elem.nativeElement) {
            return;
        }
        this.down = true;
        this.moved = false;
        this.x = (e.touches ? e.touches[0].pageX : e.pageX);
        this.y = (e.touches ? e.touches[0].pageY : e.pageY);
        this.top = this.elem.nativeElement.scrollTop;
        this.left = this.elem.nativeElement.scrollLeft;
        this.touch = true;
        // e.preventDefault(); // prevents click/hit from working
        // e.stopPropagation();
    }

    @HostListener('mousedown', ['$event']) onMousedown = (e: any) => {
        if (this.touch) {
            return;
        }
        this.down = true;
        this.moved = false;
        this.x = (e.touches ? e.touches[0].pageX : e.pageX);
        this.y = (e.touches ? e.touches[0].pageY : e.pageY);
        this.top = this.elem.nativeElement.scrollTop;
        this.left = this.elem.nativeElement.scrollLeft;
        // console.log('down', this.y);
    }

    onClick = (e) => {
        if (this.moved) {
            e.stopPropagation();
            e.preventDefault();
        }
    }
    onTouchmove = (e: any) => {
        if (this.down) {
            // this.elem.nativeElement.style.backgroundColor = "#ff00ff";
            const newX = (e.touches ? e.touches[0].pageX : e.pageX);
            const newY = (e.touches ? e.touches[0].pageY : e.pageY);
            const dx = Math.abs(this.x - newX);
            const dy = Math.abs(this.y - newY);
            // console.log('move', this.y, newY, dy, this.top);
            if (!this.moved) {
                if (dx < RSDragDirective.dragMinimum && dy < RSDragDirective.dragMinimum) {
                    return;
                }
            }
            const newTop = Math.round(this.top - newY + this.y);
            const newLeft =  Math.round(this.left - newX + this.x);
            if (this.elem.nativeElement.scrollTop !== newTop) {
              this.elem.nativeElement.scrollTop = newTop;
            }
            if (this.elem.nativeElement.scrollLeft !== newLeft) {
              this.elem.nativeElement.scrollLeft = newLeft;
            }
            this.moved = true;
        }
    }

    onMousemove = (e: any) => {
        if (this.touch) {
            return;
        }
        if (this.down) {
            // this.elem.nativeElement.style.backgroundColor = "#0000ff";
            const newX = (e.touches ? e.touches[0].pageX : e.pageX);
            const newY = (e.touches ? e.touches[0].pageY : e.pageY);
            const dx = Math.abs(this.x - newX);
            const dy = Math.abs(this.y - newY);
            if (!this.moved) {
                if (dx < RSDragDirective.dragMinimum && dy < RSDragDirective.dragMinimum) {
                    return;
                }
            }
            this.elem.nativeElement.scrollTop = (this.top - newY + this.y);
            this.elem.nativeElement.scrollLeft = (this.left - newX + this.x);
            this.moved = true;
        }
    }

    onMouseup = (e) => {
        if (this.moved) {
            RSDragDirective.lastDragEnd = new Date().getTime();
            this.moved = false;
        }
        this.down = false;
        this.touch = false;
        this.elem.nativeElement.style.backgroundColor = "transparent";
    }
}
